@@ -32,6 +32,7 @@ from mvpa2.datasets import Dataset
 from mvpa2.mappers.base import ChainMapper
 from mvpa2.mappers.zscore import zscore, ZScoreMapper
 from mvpa2.mappers.staticprojection import StaticProjectionMapper
+from mvpa2.mappers.svd import SVDMapper
 
 from mvpa2.support.due import due, Doi
 
@@ -130,6 +131,12 @@ class Hyperalignment(ClassWithCollections):
             `None` (default) an instance of
             :class:`~mvpa2.mappers.procrustean.ProcrusteanMapper` is
             used.""")
+    output_dim = Parameter(None, constraints=(EnsureInt() & EnsureRange(min=1)| EnsureNone()),
+            doc="""Output common space dimensionality. If None, datasets are aligned
+             to the features of the `ref_ds`. Otherwise, dimensionality reduction is
+             performed using SVD and only the top SVs are kept. To get all features in
+             SVD-aligned space, give output_dim>=nfeatures.
+            """)
 
     alpha = Parameter(1, constraints=EnsureFloat() & EnsureRange(min=0, max=1),
             doc="""Regularization parameter to traverse between (Shrinkage)-CCA
@@ -147,6 +154,11 @@ class Hyperalignment(ClassWithCollections):
                 reference.  If `None`, then the dataset with the maximum
                 number of features is used.""")
 
+    nproc = Parameter(1, constraints=EnsureInt(),
+            doc="""Number of processes to use to parallelize the last step of
+                alignment. If different from 1, it passes it as n_jobs to
+                `joblib.Parallel`. Requires joblib package.""")
+
     zscore_all = Parameter(False, constraints='bool',
             doc="""Flag to Z-score all datasets prior hyperalignment.
             Turn it off if Z-scoring is not desired or was already performed.
@@ -175,10 +187,22 @@ class Hyperalignment(ClassWithCollections):
             updated common space, and is subsequently called again after each
             2nd-level iteration.""")
 
+    joblib_backend = Parameter(None, constraints=EnsureChoice('multiprocessing',
+                                                    'threading') | EnsureNone(),
+            doc="""Backend to use for joblib when using nproc>1.
+            Options are 'multiprocessing' and 'threading'. Default is to use
+            'multiprocessing' unless run on OSX which have known issues with
+            joblib v0.10.3. If it is set to specific value here, then that will
+            be used at the risk of failure.""")
 
     def __init__(self, **kwargs):
         ClassWithCollections.__init__(self, **kwargs)
         self.commonspace = None
+        # mapper to a low-dimensional subspace derived using SVD on training data
+        # Initializing here so that call can access it without passing after train.
+        # Moreover, it is similar to commonspace, in that, it is required for mapping
+        # new subjects
+        self._svd_mapper = None
 
 
     @due.dcite(
@@ -198,6 +222,11 @@ class Hyperalignment(ClassWithCollections):
         """
         params = self.params            # for quicker access ;)
         ca = self.ca
+        # Check to make sure we get a list of datasets as input.
+        if not isinstance(datasets, (list, tuple, np.ndarray)):
+            raise TypeError("Input datasets should be a sequence "
+                            "(of type list, tuple, or ndarray) of datasets.")
+
         ndatasets = len(datasets)
         nfeatures = [ds.nfeatures for ds in datasets]
         alpha = params.alpha
@@ -262,23 +291,32 @@ class Hyperalignment(ClassWithCollections):
                       "it is of a floating type")
             commonspace = commonspace.astype(float)
             zscore(commonspace, chunks_attr=None)
+        # If there is only one dataset in training phase, there is nothing to be done
+        # just use that data as the common space
+        if len(datasets) < 2:
+            self.commonspace = commonspace
+        else:
+            # create a mapper per dataset
+            # might prefer some other way to initialize... later
+            mappers = [deepcopy(params.alignment) for ds in datasets]
 
-        # create a mapper per dataset
-        # might prefer some other way to initialize... later
-        mappers = [deepcopy(params.alignment) for ds in datasets]
-
-        #
-        # Level 1 -- initial projection
-        #
-        lvl1_projdata = self._level1(datasets, commonspace, ref_ds, mappers,
-                                     residuals)
-        #
-        # Level 2 -- might iterate multiple times
-        #
-        # this is the final common space
-        self.commonspace = self._level2(datasets, lvl1_projdata, mappers,
-                                        residuals)
-
+            #
+            # Level 1 -- initial projection
+            #
+            lvl1_projdata = self._level1(datasets, commonspace, ref_ds, mappers,
+                                         residuals)
+            #
+            # Level 2 -- might iterate multiple times
+            #
+            # this is the final common space
+            self.commonspace = self._level2(datasets, lvl1_projdata, mappers,
+                                            residuals)
+        if params.output_dim is not None:
+            mappers = self._level3(datasets)
+            self._svd_mapper = SVDMapper()
+            self._svd_mapper.train(self._map_and_mean(datasets, mappers))
+            self._svd_mapper = StaticProjectionMapper(
+                proj=self._svd_mapper.proj[:, :params.output_dim])
 
     def __call__(self, datasets):
         """Derive a common feature space from a series of datasets.
@@ -293,6 +331,11 @@ class Hyperalignment(ClassWithCollections):
         """
         if self.commonspace is None:
             self.train(datasets)
+        else:
+            # Check to make sure we get a list of datasets as input.
+            if not isinstance(datasets, (list, tuple, np.ndarray)):
+                raise TypeError("Input datasets should be a sequence "
+                                "(of type list, tuple, or ndarray) of datasets.")
 
         # place datasets into a copy of the list since items
         # will be reassigned
@@ -326,14 +369,14 @@ class Hyperalignment(ClassWithCollections):
             # We need to construct new mappers which would chain
             # zscore and then final transformation
             if params.alpha < 1:
-                return [ChainMapper([zm, wm, m]) for zm, wm, m in zip(zmappers, wmappers, mappers)]
+                mappers = [ChainMapper([zm, wm, m]) for zm, wm, m in zip(zmappers, wmappers, mappers)]
             else:
-                return [ChainMapper([zm, m]) for zm, m in zip(zmappers, mappers)]
-        else:
-            if params.alpha < 1:
-                return [ChainMapper([wm, m]) for wm, m in zip(wmappers, mappers)]
-            else:
-                return mappers
+                mappers = [ChainMapper([zm, m]) for zm, m in zip(zmappers, mappers)]
+        elif params.alpha < 1:
+            mappers = [ChainMapper([wm, m]) for wm, m in zip(wmappers, mappers)]
+        if params.output_dim is not None:
+            mappers = [ChainMapper([m, self._svd_mapper]) for m in mappers]
+        return mappers
 
 
     def _regularize(self, datasets, alpha):
@@ -430,7 +473,7 @@ class Hyperalignment(ClassWithCollections):
                 # common space is updated for the next iteration
                 del ds_new.sa[m.get_space()]
                 # obtain the 2nd-level projection
-                ds_ =  m.forward(ds_new.samples)
+                ds_ = m.forward(ds_new.samples)
                 if params.zscore_common:
                     zscore(ds_, chunks_attr=None)
                 # store for 2nd-level combiner
@@ -456,26 +499,97 @@ class Hyperalignment(ClassWithCollections):
 
         # key different from level-2; the common space is uniform
         #temp_commonspace = commonspace
-
-        residuals = None
-        if self.ca['residual_errors'].enabled:
-            residuals = np.zeros((1, len(datasets)))
-            self.ca.residual_errors = Dataset(samples=residuals)
+        # Fixing nproc=0
+        if params.nproc == 0:
+            from mvpa2.base import warning
+            warning("nproc of 0 doesn't make sense. Setting nproc to 1.")
+            params.nproc = 1
+        # Checking for joblib, if not, set nproc to 1
+        if params.nproc != 1:
+            from mvpa2.base import externals, warning
+            if not externals.exists('joblib'):
+                warning("Setting nproc different from 1 requires joblib package, which "
+                        "does not seem to exist. Setting nproc to 1.")
+                params.nproc = 1
 
         # start from original input datasets again
-        for i, (m, ds_new) in enumerate(zip(mappers, datasets)):
+        if params.nproc == 1:
+            residuals = []
+            for i, (m, ds_new) in enumerate(zip(mappers, datasets)):
+                if __debug__:
+                    debug('HPAL_', "Level 3: ds #%i" % i)
+                m, residual = get_trained_mapper(ds_new, self.commonspace, m,
+                                                 self.ca['residual_errors'].enabled)
+                if self.ca['residual_errors'].enabled:
+                    residuals.append(residual)
+        else:
             if __debug__:
-                debug('HPAL_', "Level 3: ds #%i" % i)
+                debug('HPAL_', "Level 3: Using joblib with nproc = %d " % params.nproc)
+            verbose_level_parallel = 20 \
+                if (__debug__ and 'HPAL' in debug.active) else 0
+            from joblib import Parallel, delayed
+            import sys
+            # joblib's 'multiprocessing' backend has known issues of failure on OSX
+            # Tested with MacOS 10.12.13, python 2.7.13, joblib v0.10.3
+            if params.joblib_backend is None:
+                params.joblib_backend = 'threading' if sys.platform == 'darwin' \
+                                        else 'multiprocessing'
+            res = Parallel(
+                    n_jobs=params.nproc, pre_dispatch=params.nproc,
+                    backend=params.joblib_backend,
+                    verbose=verbose_level_parallel
+                    )(
+                        delayed(get_trained_mapper)
+                        (ds, self.commonspace, mapper, self.ca['residual_errors'].enabled)
+                        for ds, mapper in zip(datasets, mappers)
+                    )
+            mappers = [m for m, r in res]
+            if self.ca['residual_errors'].enabled:
+                residuals = [r for m, r in res]
 
-            # retrain mapper on final common space
-            ds_new.sa[m.get_space()] = self.commonspace
-            m.train(ds_new)
-            # remove common space attribute again to save on memory
-            del ds_new.sa[m.get_space()]
-
-            if residuals is not None:
-                # obtain final projection
-                data_mapped = m.forward(ds_new.samples)
-                residuals[0, i] = np.linalg.norm(data_mapped - self.commonspace)
+        if self.ca['residual_errors'].enabled:
+            self.ca.residual_errors = Dataset(samples=np.array(residuals)[None, :])
 
         return mappers
+
+    def _map_and_mean(self, datasets, mappers):
+        params = self.params
+        data_mapped = [[] for ds in datasets]
+        for i, (m, ds_new) in enumerate(zip(mappers, datasets)):
+            if __debug__:
+                debug('HPAL_', "Mapping training data for SVD: ds #%i" % i)
+            ds_ = m.forward(ds_new.samples)
+            # XXX should we zscore data before averaging and running SVD?
+            # zscore(ds_, chunks_attr=None)
+            data_mapped[i] = ds_
+        dss_mean = params.combiner2(data_mapped)
+        return dss_mean
+
+
+def get_trained_mapper(ds, commonspace, mapper, compute_residual=False):
+    """
+    Trains a given mapper using dataset and commonspace and computes residuals if
+    necessary.
+
+    Parameters
+    ----------
+    ds: dataset
+        A dataset
+    commonspace: ndarray
+        Commonspace data.
+    mapper: Mapper
+        Typically ProcrusteanMapper.
+    compute_residual: bool
+        Whether to compute residuals or not. Default is False and returns None.
+    """
+    # retrain mapper on final common space
+    ds.sa[mapper.get_space()] = commonspace
+    mapper.train(ds)
+    # remove common space attribute again to save on memory
+    del ds.sa[mapper.get_space()]
+    residual = None
+    if compute_residual:
+        # obtain final projection
+        data_mapped = mapper.forward(ds.samples)
+        residual = np.linalg.norm(data_mapped - commonspace)
+    return mapper, residual
